<?php
namespace Teknisa\EST\Modules\Ajuste\Controllers;
use Zeedhi\Framework\DTO\Request\Filter;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use \Geral\Entities\Paramfilial;
use \Geral\Entities\Prodfili;
use \Teknisa\EST\Services\ValidacoesEstoqueService;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\EST\Services\InfComplService;

class LanctoestoqAjuste extends \Teknisa\EST\Controllers\TeknisaCrud {

    protected $dataSourceName = "lanctoestoq_ajuste";
    protected $parameterBagColumns = ['IDTPLANCTO','CDFILIAL'];
    protected $service;
    public $itensWithErrors = [];

    public function __construct ($crudService, $parameterBag, $service, $environment, ValidacoesEstoqueService $validacoesEstoqueService, InfComplService $infComplService)
    {
        parent::__construct($crudService,$parameterBag);
        $this->service = $service;
        $this->environment = $environment;
        $this->validacoesEstoqueService = $validacoesEstoqueService;
        $this->infComplService = $infComplService;
    }
    
    public function validaAjusteTotal(Request\Dataset $request, Response $response){
        $dataSet = $request->getDataSet();
		$rows = $dataSet->getRows();
		$nrorg = $this->environment->getNrOrgTrab();
		$row = $rows[0];
        $strData = substr($row['DTLANCMOVI'], 0, 10);
		$this->validacoesEstoqueService->validaDataLancamento(
                $row['CDFILIAL'], $strData, 'AJUSTE_IMP', 'EST', $strData
            );
        
		$custos = $this->validaLancamentoRows($rows, $response, $nrorg);
		
        $response->addDataSet(new DataSet('custos', $custos));
    }

    public function validaLancamento(Filter $request, Response $response) {

        try {
            $filter = $request->getFilterCriteria()->getConditions();
            foreach($filter as $condition){
                if($condition['columnName'] =='IDORDEMCLC'){
                    $idOrdemClc = $condition['value'];
                }elseif($condition['columnName'] =='CDFILIMOVI'){
                    $cdfilial = $condition['value'];
                }elseif($condition['columnName'] =='CDPRODMOVI'){
                    $cdproduto = $condition['value'];
                }elseif($condition['columnName'] =='DTLANCMOVI'){
                    $dtlancto = $condition['value'];
                }elseif($condition['columnName'] =='CDALMOXARIFE'){
                    $cdalmoxarife = $condition['value'];
                }elseif($condition['columnName'] =='CDLOCALESTOQ'){
                    $cdlocalestoq = $condition['value'];
                }elseif($condition['columnName'] =='NRLOTEESTQ'){
                    $nrloteestq = $condition['value'];
                }elseif($condition['columnName'] =='NRSUBLOTE'){
                    $nrsublote = $condition['value'];
                }elseif($condition['columnName'] =='IDAJUSTEIMPORT'){
                    $idajusteimport = $condition['value'];
                }
            }
            
            if(!isset($idajusteimport))
                $idajusteimport = 'N';
            
            $newParameterBagColumns = ["IDORDEMCLC", "CDFILIMOVI", "CDPRODMOVI", "DTLANCMOVI", "CDALMOXARIFE", "CDLOCALESTOQ", "NRLOTEESTQ", "NRSUBLOTE", "NRORG"];
            $conditions =  array();
            $newFilterCriteria = new FilterCriteria('itlanctoest_validacao', $conditions);
            $newFilterCriteria->setDataSourceName('itlanctoest_validacao');
            $newFilterCriteria->addCondition('IDORDEMCLC', $idOrdemClc);
            $newFilterCriteria->addCondition('CDFILIMOVI', $cdfilial);
            $newFilterCriteria->addCondition('CDPRODMOVI', $cdproduto);
            $newFilterCriteria->addCondition('DTLANCMOVI', $dtlancto);
            $newFilterCriteria->addCondition('CDALMOXARIFE', $cdalmoxarife);
            $newFilterCriteria->addCondition('CDLOCALESTOQ', $cdlocalestoq);
            $newFilterCriteria->addCondition('NRLOTEESTQ', $nrloteestq);
            $newFilterCriteria->addCondition('NRSUBLOTE', $nrsublote);
            $newFilterCriteria->addCondition('NRORG', $this->environment->getNrOrgTrab());
            $newFilterCriteria = $this->crudService->getFilterCriteriaWithNrOrg($newFilterCriteria);
            
            $this->dataSourceName = 'itlanctoest_validacao';
            $newResponse = new Response();
        
            $this->crudService->find($newResponse, $newFilterCriteria);
            if(isset($newResponse->getDataSets()[0]->getRows()[0])){
                $itemsImplantados = $newResponse->getDataSets()[0]->getRows()[0];
                if($idOrdemClc !== 'U'){
                    $ordem = 'primeira';
                }else{
                    $ordem = 'última';
                }

                $message = "Operação bloqueada! Implantação como $ordem operação do dia já realizada para o produto.";

                if($idajusteimport != 'S')
                    $response->setError(new Error($message, 500));
                else{
                    $response->addDataSet(new DataSet('itemsImplantados', $itemsImplantados));
                    
                    $custo = $this->service->validaCustoAjuste($cdfilial,$cdproduto,$dtlancto, $cdalmoxarife, $cdlocalestoq, $nrloteestq, $nrsublote, $idOrdemClc);
                
                    if($custo == 0 && $idajusteimport != 'S')
                        $response->setError(new Error('Produto com custo de estoque zerado.', 500));
                    
                    if(!is_array($custo))
                        $custo = array();
                        
                    $response->addDataSet(new DataSet('custo', $custo));
                }
            }else {
                $custo = $this->service->validaCustoAjuste($cdfilial,$cdproduto,$dtlancto, $cdalmoxarife, $cdlocalestoq, $nrloteestq, $nrsublote, $idOrdemClc);
                
                if($custo == 0 && $idajusteimport != 'S')
                    $response->setError(new Error('Produto com custo de estoque zerado.', 500));
                
                if(!is_array($custo))
                    $custo = array();
                    
                $response->addDataSet(new DataSet('custo', $custo));
                
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
        $this->dataSourceName = 'lanctoestoq_ajuste';
    }
    
    public function validaLancamentoRows($rows, $response, $nrorg) {

        try {
            foreach($rows as &$row){
                $idOrdemClc     = $row['IDORDEMCLC'];
                $cdfilial       = $row['CDFILIMOVI'];
                $cdproduto      = $row['CDPRODMOVI'];
                $dtlancto       = $row['DTLANCMOVI'];
                $cdalmoxarife   = $row['CDALMOXARIFE'];
                $cdlocalestoq   = $row['CDLOCALESTOQ'];
                $nrloteestq     = $row['NRLOTEESTQ'];
                $nrsublote      = $row['NRSUBLOTE'];
                
                $newParameterBagColumns = ["IDORDEMCLC", "CDFILIMOVI", "CDPRODMOVI", "DTLANCMOVI", "CDALMOXARIFE", "CDLOCALESTOQ", "NRLOTEESTQ", "NRSUBLOTE", "NRORG"];
	            $conditions =  array();
	            $newFilterCriteria = new FilterCriteria('itlanctoest_validacao', $conditions);
	            $newFilterCriteria->setDataSourceName('itlanctoest_validacao');
	            $newFilterCriteria->addCondition('IDORDEMCLC', $idOrdemClc);
	            $newFilterCriteria->addCondition('CDFILIMOVI', $cdfilial);
	            $newFilterCriteria->addCondition('CDPRODMOVI', $cdproduto);
	            $newFilterCriteria->addCondition('DTLANCMOVI', $dtlancto);
	            $newFilterCriteria->addCondition('CDALMOXARIFE', $cdalmoxarife);
	            $newFilterCriteria->addCondition('CDLOCALESTOQ', $cdlocalestoq);
	            $newFilterCriteria->addCondition('NRLOTEESTQ', $nrloteestq);
	            $newFilterCriteria->addCondition('NRSUBLOTE', $nrsublote);
	            $newFilterCriteria->addCondition('NRORG', $nrorg);
	            $newFilterCriteria = $this->crudService->getFilterCriteriaWithNrOrg($newFilterCriteria);

	            $newResponse = new Response;
	            $this->crudService->find($newResponse, $newFilterCriteria);
	            
                if(isset($newResponse->getDataSets()[0]->getRows()[0])){
                    $row['IMPLANTADOMESMODIA'] = true;
                }else {
                    $row['IMPLANTADOMESMODIA'] = "false";
                    $custo = $this->service->validaCustoAjuste($cdfilial,$cdproduto,$dtlancto, $cdalmoxarife, $cdlocalestoq, $nrloteestq, $nrsublote, $idOrdemClc);
                    
                    
                    if($custo == 0){
                        $row['VALRANTE']    = 0;
                        $row['QTDEANTE']    = 0;
                        $row['VALRMEDI']    = 0;
                        $row['VLANTERIOR']  = 0;
                        $row['SEMCUSTO']    = true;
                    }
                    
                    if(is_array($custo)){
                        $row['SEMCUSTO']    = "false";
                        $row['VALRANTE']    = $custo['valrAnte'];
                        $row['QTDEANTE']    = $custo['qtdeAnte'];
                        $row['VALRMEDI']    = $custo['valrMedi'];
                        $row['VLANTERIOR']  = $custo['vlAnterior'];
                    }
                }
            } // end loop
            return $rows;
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
            throw $e;
        }
        $this->dataSourceName = 'lanctoestoq_ajuste';
    }

    public function importProducts(Filter $request, Response $response){
        try {
            error_reporting(E_ALL);
            ini_set('display_errors', 1);
            $params = $request->getParameters();
            $tela = isset($params['origin'])? $params['origin']['containerLabel'] : 'ajuste';
            $newResponse = new Response();
            $params  = $request->getParameters();
            foreach($params['filter'] as $rowFilter){
                if($rowFilter['name'] == 'CDFILIAL')
                    $filial = $rowFilter['value'];
            }

            $this->parameterBagColumns = ['CDGRUPPROD', 'CDORDENA', 'CDFILIAL', 'CDALMOXARIFE', 'CDLOCALESTOQ', 'NRLOTEESTQ', 'NRSUBLOTE', 'IDORDEMCLC', 'DTLANCMOVI', 'IDGRUPPROD', 'CDSEGMER'];
            $filterCriteria = $request->getFilterCriteria();
            $cdSegMerc = $this->service->getCdSegMercFilial($filial);
            $filterCriteria->addCondition('CDSEGMER', $cdSegMerc);
            
            $paramFilial = $this->service->getParamFilial($filial);
            if($paramFilial->getIdutilalmox() == 'S'){
                $codAlmox = $paramFilial->getCdalmoxarife();
                if(is_null($codAlmox))
                    throw new \Exception("Unidade Controla almoxarifado e não existe almoxarifado padrão parametrizado.",400);
            }
            if($paramFilial->getIdutillcestq() == 'S'){
                $codLocal = $paramFilial->getCdlocalestoq();
                if(is_null($codLocal))
                    throw new \Exception("Unidade Controla localização e não existe local padrão parametrizado.",400);
            }
            
            if($tela == 'Implantação de Estoque'){
                $this->dataSourceName = 'implantacao_import_products';
                array_push($this->parameterBagColumns, 'CUSTLIQU');
                array_push($this->parameterBagColumns, 'CUSTBRUT');
                $paramCustos = $this->service->getParamCustos($filial);
                $filterCriteria->addCondition('CUSTLIQU', $paramCustos['CUSTLIQU']);
                $filterCriteria->addCondition('CUSTBRUT', $paramCustos['CUSTBRUT']);
            }else{
                $estNeg = $paramFilial->getIdpermestneg();
                if($estNeg == 'S')
                    $this->dataSourceName = 'ajuste_import_products_neg';
                else
                    $this->dataSourceName = 'ajuste_import_products';
            }
            
            parent::find($request, $newResponse);
            
            $arrReponse = count($newResponse->getDataSets()) > 0? $newResponse->getDataSets()[0]->getRows() : $newResponse->getDataSets();
            if($tela == 'Implantação de Estoque'){
                if(count($arrReponse) > 0 && $arrReponse[0]['FILIUTLOTE'] == 'S' && $arrReponse[0]['IDENTRAUTLOTE'] == 'S'){
                    $requestFilter =  $request->getFilterCriteria()->getConditions();
                    foreach($requestFilter as $rowFilter){
                        if($rowFilter['columnName'] == 'DTLANCMOVI')
                            $dtlanc = $rowFilter['value'];
                    }
                    foreach($arrReponse as $key => &$rowar){
                        if($rowar['UTLOTE'] == 'S'){
                            try{
                                $lote = $this->infComplService->executaProcedurePrxLoteProdEst($rowar['CDFILIAL'], $dtlanc, $rowar['CDPRODMOVI']);
                                if(is_null($lote)){
                                    throw new \Exception("Unidade Controla lote e não existe minemonicos parametrizados",400);
                                }
                                $rowar['NRLOTEESTQ'] = $lote;
                            }
                            catch (\Exception $e) {
                                $response->setError(new Error($e->getMessage(), 500));
                            }
                        }
                    }
                }
            }
            /*
            //Caso futuramente decida-se validar se todos produtos possuem custo no momento da importação, descomentar e terminar o código abaixo (este processo aumentara significamente o tempo de respota desta request)
            $requestFilter =  $request->getFilterCriteria()->getConditions();
            $dtlanc = '';
            foreach($requestFilter as $rowFilter){
                if($rowFilter['columnName'] == 'DTLANCMOVI')
                    $dtlanc = $rowFilter['value'];
            }
            
            $products = $newResponse->getDataSets()[0]->getRows();
            
            $itensImplantacao = [];
            $itensAjuste = [];
            
            foreach($products as &$produto){
                $produto["CUSTO"] = $this->service->validaCustoAjuste($produto["CDFILIAL"], $produto["CDPRODUTO"], $dtlanc, $produto["CDALMOXARIFE"], $produto["CDLOCALESTOQ"], $produto["NRLOTEESTQ"], $produto["NRSUBLOTE"], null);
                
                if($produto["CUSTO"] == 0){
                    $itensImplantacao[] = $produto;
                    $this->service->insertLoteProduto($produto);
                } else{
                    $itensAjuste[] = $produto;
                }
            }*/
            $response->addDataSet(new DataSet("products", $arrReponse));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }

        $this->dataSourceName = 'lanctoestoq_ajuste';
        $this->parameterBagColumns = ['IDTPLANCTO','CDFILIAL'];
    }

    public function delete(Request $request, Response $response){
        try{
            $row = $request->getRow();
            $this->service->deleteLancto($row['CDFILIAL'],$row['NRLANCESTQ']);
        }catch(\Exception $e){
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function importProductsPorFilial(Filter $request, Response $response) {
        try {
            $this->parameterBagColumns = [ 'CDORDENA', 'P_CDFILIAL', 'P_DATAPROC', 'DTLANCESTQ', 'CDSEGMER'];
            $params = $request->getParameters();
            $tela = isset($params['origin'])? $params['origin']['containerLabel'] : 'ajuste';
            foreach($params['filter'] as $rowFilter){
                if($rowFilter['name'] == 'P_CDFILIAL')
                    $filial = $rowFilter['value'];
            }
            
            $filterCriteria = $request->getFilterCriteria();
            $cdSegMerc = $this->service->getCdSegMercFilial($filial);
            $filterCriteria->addCondition('CDSEGMER', $cdSegMerc);
            
            $paramFilial = $this->service->getParamFilial($filial);
            if($paramFilial->getIdutilalmox() == 'S'){
                $codAlmox = $paramFilial->getCdalmoxarife();
                if(is_null($codAlmox))
                    throw new \Exception("Unidade Controla almoxarifado e não existe almoxarifado padrão parametrizado.",400);
            }
            if($paramFilial->getIdutillcestq() == 'S'){
                $codLocal = $paramFilial->getCdlocalestoq();
                if(is_null($codLocal))
                    throw new \Exception("Unidade Controla localização e não existe local padrão parametrizado.",400);
            }
            
            if($tela == 'Implantação de Estoque'){
                $this->dataSourceName = 'implantacao_import_products_filial';
                array_push($this->parameterBagColumns, 'CUSTLIQU');
                array_push($this->parameterBagColumns, 'CUSTBRUT');
                $paramCustos = $this->service->getParamCustos($filial);
                $filterCriteria->addCondition('CUSTLIQU', $paramCustos['CUSTLIQU']);
                $filterCriteria->addCondition('CUSTBRUT', $paramCustos['CUSTBRUT']);
            }else{
                $estNeg = $paramFilial->getIdpermestneg();
                if($estNeg == 'S')
                    $this->dataSourceName = 'ajuste_import_products_filial_neg';
                else    
                    $this->dataSourceName = 'ajuste_import_products_filial';
            }
            
            
            $newResponse = new Response();
            
            parent::find($request, $newResponse);
            
            $arrReponse = count($newResponse->getDataSets()) > 0? $newResponse->getDataSets()[0]->getRows() : $newResponse->getDataSets();
            if($tela == 'Implantação de Estoque'){
                if(count($arrReponse) > 0 && $arrReponse[0]['FILIUTLOTE'] == 'S'){
                    
                    $requestFilter =  $request->getFilterCriteria()->getConditions();
                    foreach($requestFilter as $rowFilter){
                        if($rowFilter['columnName'] == 'DTLANCMOVI' || $rowFilter['columnName'] == 'DTLANCESTQ')
                            $dtlanc = $rowFilter['value'];
                    }
                    foreach($arrReponse as $key => &$rowar){
                        if($rowar['UTLOTE'] == 'S'){
                            try{
                                $lote = $this->infComplService->executaProcedurePrxLoteProdEst($rowar['CDFILIAL'], $dtlanc, $rowar['CDPRODMOVI']);
                                if(is_null($lote)){
                                    throw new \Exception("Unidade Controla lote e não existe minemonicos parametrizados",400);
                                }
                                $rowar['NRLOTEESTQ'] = $lote;
                            }   
                            catch (\Exception $e) {
                                $response->setError(new Error($e->getMessage(), 500));
                            }
                        }
                    }
                }
            }
            
            $response->addDataSet(new DataSet("products", $arrReponse));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }

        $this->dataSourceName = 'lanctoestoq_ajuste';
        $this->parameterBagColumns = ['IDTPLANCTO','CDFILIAL'];
    }
    
    public function importarProdutosTxt(Row $request, Response $response){
        $row = $request->getRow();
        
        //verifies if the importation type is txt or xls/xlsx
        $isTxt = strpos($row['UPLOADFILE'][0]['type'],'text/plain') !== false;
        if($isTxt){
            $arquivoB64 = $row["UPLOADFILE"][0]["b64File"];
            if(strpos($arquivoB64, ",") !== false){
                $arquivoB64 = explode(",", $arquivoB64)[1];
            }
            $arquivo = utf8_encode(base64_decode($arquivoB64));
            $produtos = explode("\n", $arquivo);
            $produtos = array_filter($produtos, function($v){
                return $v != "" && $v != null; 
            });
        }else{
            $table = $this->service->readXls($row);
            $produtos = $this->service->formatXls($table);
        }
        try{
            $params = $row["PARAMS"];
            $itensImplantacao = [];
            $itensAjuste = [];
            $boolImplantacao = $row['CONTAINER'] == 'est42000_implantacao_estoque'? true: false;
            $ErrorRowNumber = 1;
            
            foreach($produtos as &$produto){
                if($isTxt)
                    $produto = $this->service->buildObjectTheCode($produto, $params, $boolImplantacao,$this->itensWithErrors,$ErrorRowNumber);
                else
                    $produto = $this->service->buildObjectTheCodeXls($produto, $params, $boolImplantacao,$this->itensWithErrors,$ErrorRowNumber);
                    
                $produto = $this->service->completaProduto($produto, $row['CDFILIAL'], $row["CDFILIMOVI"], $row["DTLANCESTQ"],$this->itensWithErrors,$ErrorRowNumber);
                if(($produto["CUSTO"] == 0 || (isset($produto["CUSTO"]["valrMedi"])? $produto["CUSTO"]["valrMedi"] == 0 : false )) || $row['CONTAINER'] == 'est42000_implantacao_estoque'){
                    $itensImplantacao[] = $produto;
                    $this->service->insertLocaisEstoque($produto);
                } else{
                    $itensAjuste[] = $produto;
                }
                $ErrorRowNumber++;
            }
            $this->dataSourceName = 'lanctoestoq_ajuste';
            $response->addDataSet(new DataSet("itensAjuste", $itensAjuste));
            $response->addDataSet(new DataSet("itensImplantacao", $itensImplantacao));
        } catch(\Exception $e) {
            $response->addDataSet(new DataSet("itensWithErrors", $this->itensWithErrors));
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
}