<?php
namespace Teknisa\EST\Modules\EstoqueCego\Services;

use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;

class Est90000PerfilDeOperadoresDoInventarioCegoService {
    
    protected $Factory;
    protected $dataSourceManager;
    
    public function __construct($Est90000factory, $environment,$dataSourceManager) {
        $this->Factory = $Est90000factory;
        $this->dataSourceManager = $dataSourceManager;
        $this->environment = $environment;
    }
    
    public function onSaveOperation ($row){
        $groupsRequired = $this->ableGroupsArrayBuilder($row);
        $ableGroups = $this->ableGroupsArrayBuilder($this->Factory->getAbleGroups(),true);
        
        $MissesGroup = $this->checkHasValuesArray($groupsRequired,$ableGroups);
        
        if(!empty($MissesGroup)){
            throw new \Exception('Operação bloqueada! Não existe grupo de operadores associado ao perfil "'.$MissesGroup.'". Defina o grupo associado na tela "Parâmetros de Estoque", página "Inventário Cego".');
        }
        
        foreach($row as $field=>$value){
            switch($field){
                case 'IDGEST':
                case 'IDGGER':
                case 'IDGAUD':
                    $this->checkSaveOperation($row['CDOPERADOR'],$field,$value);
                    $this->checkMenu($row['CDOPERADOR'],$field);
                    break;
                default:
                    break;
            }
        }
    }
    
    public function checkSaveOperation($CDOPERADOR,$field,$value){
        
        $factoryReturn = $this->Factory->checkField($CDOPERADOR,$field);
        
        $params = [ 
            ":CDGRUPOPER" => $this->Factory->getCDGRUPOPER(substr($field, -4)),
            ":CDOPERADOR" => $CDOPERADOR
        ];
        if($value === 'S' && empty($factoryReturn)){
            $this->Factory->saveOperation($params);
            
        }else if($value === 'N' && !empty($factoryReturn)){
            $this->Factory->deleteOperation($params);
        }
    }
    
    public function checkMenu($CDOPERADOR,$field){
        $params = [ 
            ":CDGRUPOPER" => $this->Factory->getCDGRUPOPER(substr($field, -4)),
            ":CDOPERADOR" => $CDOPERADOR
        ];
            
        $factoryReturn = $this->Factory->checkField($CDOPERADOR,$field);
        
        if(!empty($factoryReturn)){
            $params[":IDMENUPRINOP"] = "S";
            $this->Factory->updateMenu($params);
            
        }else{
            $params[":IDMENUPRINOP"] = "N";
            $this->Factory->updateMenu($params);
            
        }
    }
    
    public function onDeleteOperation($row){
        foreach($row as $field=>$value){
                switch($field){
                    case 'IDGEST':
                    case 'IDGGER':
                    case 'IDGAUD':
        
                        $params = [ 
                            ":CDGRUPOPER" => $this->Factory->getCDGRUPOPER(substr($field, -4)),
                            ":CDOPERADOR" => $row['CDOPERADOR']
                        ];
                        $this->Factory->deleteOperation($params);
                        $this->checkMenu($row['CDOPERADOR'],$field);
                        break;
                    default:
                        break;
                }
            }
    }
    
    public function ableGroupsArrayBuilder($row,$fromQuery = false){
        $ableGroupsArray = [];
        if($fromQuery){
            foreach($row as $group){
                foreach($group as $value){
                    if($value === 'GEST' || $value === 'GGER' || $value === 'GAUD' ){
                        array_push($ableGroupsArray,$value);
                    }
                };
            };
        }else{
            foreach($row as $field=>$value){
                if($value === "S"){
                    if($field === 'IDGEST' || $field === 'IDGGER' || $field === 'IDGAUD' ){
                        array_push($ableGroupsArray,substr($field,-4));
                    }
                }
            }
        }
        return $ableGroupsArray;
    }
    
    public function checkHasValuesArray($FirstArr, $SecondArr){
        foreach($FirstArr as $FirstVal){
            if (!in_array($FirstVal, $SecondArr)) {
                switch($FirstVal){
                    case 'GEST':
                        return 'Estoquista';
                    case 'GGER':
                        return 'Gerente';
                    case 'GAUD':
                        return 'Auditor';
                    default:
                        break;
                }
            }
        }
        return "";
    }
}
