module.exports = function(grunt) {
	var srcPath = [
		'src/services/ZhLayoutPreferences.js',
		'src/controllers/ZhLayoutWidgetsBuilder.js',
		'src/controllers/ZhLayoutCreateLayoutOptionsController.js',
		'src/controllers/ZhLayoutCreateLayoutController.js',
		'src/controllers/ZhLayoutImportLayoutController.js',
		'src/controllers/ZhLayoutConfigController.js',
		'src/controllers/ZhLayoutFloatingCardController.js',
		'src/directives/ZhLayoutFloatingControl.js',
		'src/Config.js'
	];

	var codeCoverageFiles = [srcPath, '!src/directives/ZhLayoutFloatingControl.js', '!src/Config.js'];

	var dependencies = ['node_modules/q/q.js', 'bower_components/zeedhi-frontend/assets/dist/main.js'];
	var helpers = ['tests/helpers/**/*.js'];
	var specsPath = 'tests/specs/**/*.spec.js';

	grunt.initConfig({
		jshint: {
			all: ['GruntFile.js', srcPath, specsPath]
		},
		jasmine: {
			pivotal: {
				src: codeCoverageFiles,
				options: {
					specs: specsPath,
					vendor: dependencies,
					helpers: helpers,
					template: require('grunt-template-jasmine-istanbul'),
					templateOptions: {
						coverage: 'bin/coverage/coverage.json',
						report: 'bin/coverage',
						thresholds: {
							lines: 100,
							statements: 100,
							branches: 100,
							functions: 100
						}
					}
				}
			},
			toWatch: {
				src: codeCoverageFiles,
				options: {
					vendor: dependencies,
					specs: specsPath,
					helpers: helpers
				}
			}
		},
		bump: {
			options: {
				files: ['bower.json', 'package.json'],
				pushTo: 'origin',
				commitFiles: ["-a"],
				push: true
			}
		},
		concat: {
			zh: {
				options: {
					process: function(src, filepath) {
						return '// FILE: ' + filepath + '\n' + src + '\n';
					}
				},
				src: srcPath,
				dest: 'dist/layout-preferences.js'
			}
		},
		uglify: {
			options: {
				mangle: false,
				compress: false,
				report: 'min',
				banner: '/*! <%= grunt.template.today("dd-mm-yyyy") %> */\n'
			},
			dist: {
				files: {
					'dist/layout-preferences.min.js': srcPath
				}
			}
		},
		buildEnvironment: {
			development: {
				configFile: 'example/config/development.json'
			},
			deploy: {
				configFile: 'example/config/deploy.json'
			},
			srcFolders: ['example/frontend', 'example/frontend/tests']
		},
		jsdoc: {
			dist: {
				options: {
					configure: 'jsdoc.conf.json',
					recurse: true
				}
			}
		}
	});

	grunt.loadNpmTasks('grunt-bump');
	grunt.loadNpmTasks('grunt-jsdoc');
	grunt.loadNpmTasks('grunt-contrib-concat');
	grunt.loadNpmTasks('grunt-contrib-jshint');
	grunt.loadNpmTasks('grunt-contrib-jasmine');
	grunt.loadNpmTasks('grunt-contrib-uglify');
	grunt.loadNpmTasks('grunt-zh-environment-builder');
	grunt.loadNpmTasks('grunt-zh-id-generator');

	grunt.registerTask('default', ['jshint', 'jasmine:toWatch', 'concat']);
	grunt.registerTask('test', ['jshint', 'jasmine:pivotal']);
	grunt.registerTask('doc', ['jsdoc']);
	grunt.registerTask('deploy', ['concat', 'uglify']);

};